import { Node } from "./cst/Node";
type NodeByType<TType, TNode> = Extract<TNode, {
    type: TType;
}>;
/**
 * A map with a transform function for each Node type, like:
 *
 *     { select_clause: (node: SelectClause) => {},
 *       from_clause: (node: FromClause) => {},
 *       paren_expr: (node: ParenExpr) => {},
 *       ... }
 */
export type FullTransformMap<T, TNode extends Node = Node> = {
    [TType in TNode["type"]]: (node: NodeByType<TType, TNode>) => T;
};
export type TransformMap<T, TNode extends Node = Node> = Partial<FullTransformMap<T, TNode>>;
/**
 * Creates a function that transforms the whole syntax tree to data type T.
 * @param map An object with a transform function for each CST node type
 */
export declare function cstTransformer<T>(map: TransformMap<T>): (node: Node) => T;
export {};
