"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.cstVisitor = exports.VisitorAction = void 0;
const generic_1 = require("./utils/generic");
var VisitorAction;
(function (VisitorAction) {
    /** Return from visitor function to skip iterating over child nodes. */
    VisitorAction[VisitorAction["SKIP"] = 1] = "SKIP";
})(VisitorAction = exports.VisitorAction || (exports.VisitorAction = {}));
/**
 * Creates a function that visits the whole syntax tree.
 * @param map An object with a visitor functions
 */
function cstVisitor(map) {
    const visit = (node) => {
        const visitType = map[node.type];
        // Visit the node itself
        const action = visitType === null || visitType === void 0 ? void 0 : visitType(node);
        if (action !== VisitorAction.SKIP) {
            // Visit all children
            for (const child of Object.values(node)) {
                if (isNode(child)) {
                    visit(child);
                }
                else if (child instanceof Array) {
                    child.filter(isNode).forEach((childNode) => visit(childNode));
                }
            }
        }
    };
    return visit;
}
exports.cstVisitor = cstVisitor;
const isNode = (value) => (0, generic_1.isObject)(value) && (0, generic_1.isString)(value.type);
