"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exprMap = void 0;
const show_1 = require("../show");
exports.exprMap = {
    list_expr: (node) => (0, show_1.show)(node.items, ","),
    paren_expr: (node) => "(" + (0, show_1.show)(node.expr) + ")",
    binary_expr: (node) => (0, show_1.show)([node.left, node.operator, node.right]),
    prefix_op_expr: (node) => (0, show_1.show)([node.operator, node.expr]),
    postfix_op_expr: (node) => (0, show_1.show)([node.expr, node.operator]),
    func_call: (node) => (0, show_1.show)([node.name, node.args, node.filter, node.over]),
    func_args: (node) => (0, show_1.show)([
        node.distinctKw,
        node.args,
        node.nullHandlingKw,
        node.orderBy,
        node.limit,
        node.having,
    ]),
    named_arg: (node) => (0, show_1.show)([node.name, node.operator, node.value]),
    cast_expr: (node) => (0, show_1.show)([node.castKw, node.args]),
    cast_operator_expr: (node) => (0, show_1.show)([node.left, node.operator, node.right]),
    cast_arg: (node) => (0, show_1.show)([node.expr, node.asKw, node.dataType, node.format]),
    cast_format: (node) => (0, show_1.show)([node.formatKw, node.string, node.timezone]),
    cast_format_timezone: (node) => (0, show_1.show)([node.atTimeZoneKw, node.timezone]),
    row_constructor: (node) => (0, show_1.show)([node.rowKw, node.row]),
    array_constructor: (node) => (0, show_1.show)([node.arrayKw, node.expr]),
    raise_expr: (node) => (0, show_1.show)([node.raiseKw, node.args]),
    raise_expr_type: (node) => (0, show_1.show)(node.typeKw),
    extract_expr: (node) => (0, show_1.show)([node.extractKw, node.args]),
    extract_from: (node) => (0, show_1.show)([node.unit, node.fromKw, node.expr]),
    week_expr: (node) => (0, show_1.show)([node.weekKw, node.args]),
    filter_arg: (node) => (0, show_1.show)([node.filterKw, node.where]),
    over_arg: (node) => (0, show_1.show)([node.overKw, node.window]),
    having_arg: (node) => (0, show_1.show)([node.havingKw, node.minMaxKw, node.expr]),
    between_expr: (node) => (0, show_1.show)([node.left, node.betweenKw, node.begin, node.andKw, node.end]),
    case_expr: (node) => (0, show_1.show)([node.caseKw, node.expr, node.clauses, node.endKw]),
    case_when: (node) => (0, show_1.show)([node.whenKw, node.condition, node.thenKw, node.result]),
    case_else: (node) => (0, show_1.show)([node.elseKw, node.result]),
    interval_expr: (node) => (0, show_1.show)([node.intervalKw, node.expr, node.unit]),
    interval_unit_range: (node) => (0, show_1.show)([node.fromUnit, node.toKw, node.toUnit]),
    interval_unit: (node) => (0, show_1.show)([node.unitKw]),
    typed_expr: (node) => (0, show_1.show)([node.dataType, node.expr]),
    array_expr: (node) => (0, show_1.show)(["[", node.expr, "]"]),
    struct_expr: (node) => (0, show_1.show)(["(", node.expr, ")"]),
    quantifier_expr: (node) => (0, show_1.show)([node.quantifierKw, node.expr]),
    full_text_match_expr: (node) => (0, show_1.show)([node.matchKw, node.columns, node.againstKw, node.args]),
    full_text_match_args: (node) => (0, show_1.show)([node.expr, node.modifier]),
    // Tables & columns
    member_expr: (node) => (0, show_1.show)([node.object, node.property], node.property.type === "array_subscript" ? "" : "."),
    bigquery_quoted_member_expr: (node) => (0, show_1.show)(["`", node.expr, "`"]),
    array_subscript: (node) => (0, show_1.show)(["[", node.expr, "]"]),
    array_subscript_specifier: (node) => (0, show_1.show)([node.specifierKw, node.args]),
    array_slice_specifier: (node) => (0, show_1.show)([node.from, ":", node.to]),
    // special literals
    string_with_charset: (node) => "_" + node.charset + (0, show_1.show)(node.string),
    datetime_literal: (node) => (0, show_1.show)([node.datetimeKw, node.string]),
    date_literal: (node) => (0, show_1.show)([node.dateKw, node.string]),
    time_literal: (node) => (0, show_1.show)([node.timeKw, node.string]),
    timestamp_literal: (node) => (0, show_1.show)([node.timestampKw, node.string]),
    json_literal: (node) => (0, show_1.show)([node.jsonKw, node.string]),
    jsonb_literal: (node) => (0, show_1.show)([node.jsonbKw, node.string]),
    numeric_literal: (node) => (0, show_1.show)([node.numericKw, node.string]),
    bignumeric_literal: (node) => (0, show_1.show)([node.bignumericKw, node.string]),
    interval_literal: (node) => (0, show_1.show)([node.intervalKw, node.string]),
    // Basic language elements
    identifier: (node) => node.text,
    string_literal: (node) => node.text,
    number_literal: (node) => node.text,
    blob_literal: (node) => node.text,
    boolean_literal: (node) => (0, show_1.show)(node.valueKw),
    null_literal: (node) => (0, show_1.show)(node.nullKw),
    variable: (node) => node.text,
    parameter: (node) => node.text,
};
