"use strict";
//
// Helpers for parsing binary data (hex and bit strings)
//
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseTextBlob = exports.chunksFromRight = exports.parseBitBlob = exports.parseHexBlob = void 0;
/**
 * Converts hex string to decimal array
 *
 *     "5A59" -> [90, 89]
 */
const parseHexBlob = (data) => {
    return (0, exports.chunksFromRight)(data, 2).map((m) => parseInt(m, 16));
};
exports.parseHexBlob = parseHexBlob;
/**
 * Converts bit string to decimal array
 *
 *     "0101101001011001" -> [90, 89]
 */
const parseBitBlob = (data) => {
    return (0, exports.chunksFromRight)(data, 8).map((m) => parseInt(m, 2));
};
exports.parseBitBlob = parseBitBlob;
/**
 * Slices string to chunks of specified max size,
 * starting the measurement from right, like so:
 *
 *     chunksFromRight("ABCDEFG", 3) -> ["A", "BCD", "EFG"]
 */
const chunksFromRight = (text, chunkSize) => {
    let startIndex = text.length % chunkSize;
    const chunks = [];
    if (startIndex > 0) {
        chunks.push(text.slice(0, startIndex));
    }
    while (startIndex < text.length) {
        chunks.push(text.slice(startIndex, startIndex + chunkSize));
        startIndex += chunkSize;
    }
    return chunks;
};
exports.chunksFromRight = chunksFromRight;
/**
 * Converts string to decimal array
 */
const parseTextBlob = (data) => {
    // TextEncoder is globally available in NodeJS,
    // but for some reason the type is missing.
    // See: https://github.com/DefinitelyTyped/DefinitelyTyped/discussions/60038
    // eslint-disable-next-line @typescript-eslint/ban-ts-comment
    // @ts-ignore
    return [...new TextEncoder().encode(data)]; // eslint-disable-line
};
exports.parseTextBlob = parseTextBlob;
