"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createArrayDataTypeChain = exports.createListExpr = exports.createParenExpr = exports.createAlias = exports.createIdentifier = exports.createKeyword = exports.createPostfixOpExpr = exports.createPrefixOpExpr = exports.createFuncCall = exports.createMemberExprChain = exports.createJoinExprChain = exports.createCompoundSelectStmt = exports.createCompoundSelectStmtChain = exports.createCastOperatorExprChain = exports.createBinaryExpr = exports.createBinaryExprChain = exports.createStringConcatExprChain = void 0;
const whitespace_1 = require("./whitespace");
const list_1 = require("./list");
const generic_1 = require("./generic");
//
// Helper functions for creating various types of Node objects
//
const deriveLoc = (binExpr) => {
    if (!binExpr.left.range || !binExpr.right.range) {
        return binExpr;
    }
    const start = binExpr.left.range[0];
    const end = binExpr.right.range[1];
    return Object.assign(Object.assign({}, binExpr), { range: [start, end] });
};
function createStringConcatExprChain(head, tail) {
    return tail.reduce((left, [c1, right]) => deriveLoc(createBinaryExpr(left, c1, "", [], right)), head);
}
exports.createStringConcatExprChain = createStringConcatExprChain;
function createBinaryExprChain(head, tail) {
    return tail.reduce((left, [c1, op, c2, right]) => deriveLoc(createBinaryExpr(left, c1, op, c2, right)), head);
}
exports.createBinaryExprChain = createBinaryExprChain;
function createBinaryExpr(left, c1, op, c2, right) {
    return {
        type: "binary_expr",
        operator: op,
        left: (0, whitespace_1.trailing)(left, c1),
        right: (0, whitespace_1.leading)(right, c2),
    };
}
exports.createBinaryExpr = createBinaryExpr;
function createCastOperatorExprChain(head, tail) {
    return tail.reduce((left, [c1, op, c2, right]) => deriveLoc(createCastExpr(left, c1, op, c2, right)), head);
}
exports.createCastOperatorExprChain = createCastOperatorExprChain;
function createCastExpr(left, c1, op, c2, right) {
    return {
        type: "cast_operator_expr",
        left: (0, whitespace_1.trailing)(left, c1),
        operator: op,
        right: (0, whitespace_1.leading)(right, c2),
    };
}
function createCompoundSelectStmtChain(head, tail) {
    return tail.reduce((left, [c1, op, c2, right]) => deriveLoc(createCompoundSelectStmt(left, c1, op, c2, right)), head);
}
exports.createCompoundSelectStmtChain = createCompoundSelectStmtChain;
function createCompoundSelectStmt(left, c1, op, c2, right) {
    return {
        type: "compound_select_stmt",
        operator: op,
        left: (0, whitespace_1.trailing)(left, c1),
        right: (0, whitespace_1.leading)(right, c2),
    };
}
exports.createCompoundSelectStmt = createCompoundSelectStmt;
const deriveJoinLoc = (join) => {
    var _a, _b;
    if (!join.left.range) {
        return join;
    }
    const start = join.left.range[0];
    const end = (_b = (_a = (join.specification || join.right)) === null || _a === void 0 ? void 0 : _a.range) === null || _b === void 0 ? void 0 : _b[1];
    return Object.assign(Object.assign({}, join), { range: [start, end] });
};
const derivePivotlikeLoc = (pivot) => {
    if (!pivot.left.range || !pivot.args.range) {
        return pivot;
    }
    const start = pivot.left.range[0];
    const end = pivot.args.range[1];
    return Object.assign(Object.assign({}, pivot), { range: [start, end] });
};
const deriveForSystemTimeLoc = (expr) => {
    if (!expr.left.range || !expr.expr.range) {
        return expr;
    }
    const start = expr.left.range[0];
    const end = expr.expr.range[1];
    return Object.assign(Object.assign({}, expr), { range: [start, end] });
};
function createJoinExprChain(head, tail) {
    return tail.reduce((left, createNodeFn) => {
        const node = createNodeFn(left);
        switch (node.type) {
            case "join_expr":
                return deriveJoinLoc(node);
            case "pivot_expr":
                return derivePivotlikeLoc(node);
            case "unpivot_expr":
                return derivePivotlikeLoc(node);
            case "tablesample_expr":
                return derivePivotlikeLoc(node);
            case "for_system_time_as_of_expr":
                return deriveForSystemTimeLoc(node);
        }
    }, head);
}
exports.createJoinExprChain = createJoinExprChain;
function isObjectMemberExprTailPart(part) {
    return part[1] === ".";
}
function isFuncCallMemberExprTailPart(part) {
    return (0, generic_1.isObject)(part[1]) && part[1].type === "func_call_right";
}
function createMemberExprChain(head, tail) {
    return tail.reduce((obj, tailPart) => {
        if (isFuncCallMemberExprTailPart(tailPart)) {
            return deriveFuncCallLoc(createFuncCall(obj, tailPart));
        }
        else {
            return deriveMemberExprLoc(createMemberExpr(obj, tailPart));
        }
    }, head);
}
exports.createMemberExprChain = createMemberExprChain;
function createMemberExpr(obj, tailPart) {
    if (isObjectMemberExprTailPart(tailPart)) {
        const [c1, _, c2, prop] = tailPart;
        return {
            type: "member_expr",
            object: (0, whitespace_1.trailing)(obj, c1),
            property: (0, whitespace_1.leading)(prop, c2),
        };
    }
    else {
        const [c1, prop] = tailPart;
        return {
            type: "member_expr",
            object: obj,
            property: (0, whitespace_1.leading)(prop, c1),
        };
    }
}
function createFuncCall(name, [c1, fn]) {
    return {
        type: "func_call",
        name: (0, whitespace_1.trailing)(name, c1),
        args: fn.args,
        filter: fn.filter,
        over: fn.over,
    };
}
exports.createFuncCall = createFuncCall;
const deriveMemberExprLoc = (expr) => {
    if (!expr.object.range || !expr.property.range) {
        return expr;
    }
    const start = expr.object.range[0];
    const end = expr.property.range[1];
    return Object.assign(Object.assign({}, expr), { range: [start, end] });
};
const deriveFuncCallLoc = (fn) => {
    const right = fn.over || fn.filter || fn.args;
    if (!right) {
        throw new Error("Unexpected argument-less function call.");
    }
    if (!fn.name.range || !right.range) {
        return fn;
    }
    const start = fn.name.range[0];
    const end = right.range[1];
    return Object.assign(Object.assign({}, fn), { range: [start, end] });
};
function createPrefixOpExpr(op, expr) {
    return {
        type: "prefix_op_expr",
        operator: op,
        expr: expr,
    };
}
exports.createPrefixOpExpr = createPrefixOpExpr;
function createPostfixOpExpr(op, expr) {
    return {
        type: "postfix_op_expr",
        expr: expr,
        operator: op,
    };
}
exports.createPostfixOpExpr = createPostfixOpExpr;
const createKeyword = (text) => ({
    type: "keyword",
    text,
    name: text.toUpperCase(),
});
exports.createKeyword = createKeyword;
const createIdentifier = (text, name) => ({
    type: "identifier",
    text,
    name,
});
exports.createIdentifier = createIdentifier;
const createAlias = (expr, _alias) => {
    if (!_alias) {
        return expr;
    }
    const [c, partialAlias] = _alias;
    return Object.assign({ type: "alias", expr: (0, whitespace_1.trailing)(expr, c) }, partialAlias);
};
exports.createAlias = createAlias;
const createParenExpr = (c1, expr, c2) => {
    return {
        type: "paren_expr",
        expr: (0, whitespace_1.surrounding)(c1, expr, c2),
    };
};
exports.createParenExpr = createParenExpr;
const createListExpr = (head, tail) => {
    return {
        type: "list_expr",
        items: (0, list_1.readCommaSepList)(head, tail),
    };
};
exports.createListExpr = createListExpr;
function createArrayDataTypeChain(head, tail) {
    return tail.reduce((left, [c1, right]) => deriveArrayDataTypeLoc(createArrayDataType(left, c1, right)), head);
}
exports.createArrayDataTypeChain = createArrayDataTypeChain;
const deriveArrayDataTypeLoc = (array) => {
    if (!array.dataType.range || !array.bounds.range) {
        return array;
    }
    const start = array.dataType.range[0];
    const end = array.bounds.range[1];
    return Object.assign(Object.assign({}, array), { range: [start, end] });
};
function createArrayDataType(dataType, c1, bounds) {
    return {
        type: "array_data_type",
        dataType: (0, whitespace_1.trailing)(dataType, c1),
        bounds: bounds,
    };
}
