"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.read = void 0;
const whitespace_1 = require("./whitespace");
/* eslint-disable @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-unsafe-return */
/**
 * Given array of syntax nodes and whitespace or single node or null,
 * associates whitespace with syntax nodes.
 *
 * @param {(Node | Whitespace)[] | Node | null} items
 * @return {Node[] | Node | undefined}
 */
const read = (items) => {
    if (!items) {
        return undefined; // convert null to undefined
    }
    if (!(items instanceof Array)) {
        return items; // leave single syntax node as-is
    }
    // associate leading/trailing whitespace with nodes
    const nodes = [];
    let leadingWhitespace = undefined;
    for (const it of items) {
        if (isWhitespace(it)) {
            if (nodes.length > 0) {
                nodes[nodes.length - 1] = (0, whitespace_1.trailing)(nodes[nodes.length - 1], it);
            }
            else {
                leadingWhitespace = it;
            }
        }
        else {
            if (leadingWhitespace) {
                nodes.push((0, whitespace_1.leading)(it, leadingWhitespace));
                leadingWhitespace = undefined;
            }
            else {
                nodes.push(it);
            }
        }
    }
    return nodes.length > 1 ? nodes : nodes[0];
};
exports.read = read;
/* eslint-enable @typescript-eslint/no-unsafe-argument, @typescript-eslint/no-unsafe-return */
// True when dealing with whitespace array (as returned by __ rule)
const isWhitespace = (item) => {
    if (!(item instanceof Array)) {
        return false;
    }
    if (item.length === 0) {
        return true;
    }
    return Boolean(whitespaceType[item[0].type]);
};
const whitespaceType = {
    space: true,
    newline: true,
    line_comment: true,
    block_comment: true,
};
