"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.surrounding = exports.trailing = exports.leading = void 0;
const generic_1 = require("./generic");
function leading(node, ws) {
    if (node instanceof Array) {
        // Add leading whitespace to first item in array
        return setFirst(node, leading(node[0], ws));
    }
    if (!(0, generic_1.isObject)(node)) {
        throw new Error(`Expected Node object, instead got ${JSON.stringify(node)}`);
    }
    if (ws && ws.length) {
        if (node.leading) {
            throw new Error("leading(): Node already has leading whitespace");
        }
        return Object.assign(Object.assign({}, node), { leading: ws });
    }
    return node;
}
exports.leading = leading;
function trailing(node, ws) {
    if (node instanceof Array) {
        // Add trailing whitespace to last item in array
        return setLast(node, trailing((0, generic_1.last)(node), ws));
    }
    if (!(0, generic_1.isObject)(node)) {
        throw new Error(`Expected Node object, instead got ${JSON.stringify(node)}`);
    }
    if (ws && ws.length) {
        if (node.trailing) {
            throw new Error("trailing(): Node already has trailing whitespace");
        }
        return Object.assign(Object.assign({}, node), { trailing: ws });
    }
    return node;
}
exports.trailing = trailing;
function surrounding(leadingWs, node, trailingWs) {
    return trailing(leading(node, leadingWs), trailingWs);
}
exports.surrounding = surrounding;
// Creates new array with first item replaced by value
const setFirst = ([_oldFirst, ...rest], value) => {
    return [value, ...rest];
};
// Creates new array with last item replaced by value
const setLast = (array, value) => {
    const rest = array.slice(0, -1);
    return [...rest, value];
};
